/*
 * box.cpp
 *
 *  Created on: 18 May 2016
 *      Author: strazce
 */

#include <memory>

#include "data/box.h"

namespace opice {
namespace data {

using conf::Settings;
using conf::DomainSettings;

/*** Initializes the Box class which is aware of the parameters of the whole simulation box including MPI domains
 *
 *   The constructor will modify the settings object so that from this point onward, it will define the settings
 *   relevant to this Domain only. That is e.g. the Domain size won't be as defined in the configuration file, but
 *   it will be cut to fit this MPI process.
 *
 *
 */
Box::Box(Settings* settings)
    : _settings(settings),
      _old_domain_settings(new DomainSettings(_settings->domain)), ///< @internal Take care initialize old settings before cutting the domain
      _domain(cut_mpi_domain())
    {

}

std::unique_ptr<Domain> Box::cut_mpi_domain(){
    _settings->domain.size = _settings->phys_size;
    std::unique_ptr<Domain> dom(new Domain(&_settings->domain));
    return dom;
}

void Box::init_domain(){
    _domain->init();
}

} /* namespace data */
} /* namespace opice */
