/*
 * input_reader_boost.cpp
 *
 *  Created on: Feb 10, 2011
 *      Author: strazce
 */

#include "conf/input_reader_boost.h"

#include "utils/basic_logger.h"

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <cstdio>
#include <cstdlib>

//#include "boundary.h"

namespace po = boost::program_options;

using namespace opice::conf;

namespace opice {
namespace conf {


InputReaderBoost::InputReaderBoost(Settings *sett){
    _settings = sett;
}

void InputReaderBoost::read_config_file(std::string filename){
    po::options_description generic("Simulation options");
    generic.add_options()
        ("help", "produce help message")

        ("nsteps", po::value<int>(), "number of simulation time steps")

        ("phys_size", po::value<std::string>(), "number of cells in physical region x,y,z axis (vector)")


//        ("dy_factor", po::value<double>()->default_value(1), "dy factor relative to dx")
//        ("dz_factor", po::value<double>()->default_value(1), "dz factor relative to dx")
//        ("dt_factor", po::value<double>()->default_value(0.99), "dt factor relative to CFL limit")

        #ifdef MOVING_WINDOW
        ("window_size", po::value<int>()->default_value(0), "size of the moving window (x-direction)")
        ("window_initial_size", po::value<int>()->default_value(0), "initial size of the moving window (=window_size if set to 0)")
        ("window_start", po::value<int>()->default_value(0), "start moving window at this timestep")
        ("window_stop", po::value<int>()->default_value(0), "stop moving window at this timestep (0 = never)")
        #endif

        ("plasma_min", po::value<std::string>(), "min coordinates of plasma slab (vector)")
        ("plasma_max", po::value<std::string>(), "max coordinates of plasma slab (vector)")

//        ("nspheres", po::value<int>(), "number of plasma spheres")
//        ("plasma_sphere_x", po::value<double>()->default_value(0), "x coordinate of centre of spherical plasma slab")
//        ("plasma_sphere_y", po::value<double>()->default_value(0), "y coordinate of centre of spherical plasma slab")
//        ("plasma_sphere_z", po::value<double>()->default_value(0), "z coordinate of centre of spherical plasma slab")
//        ("plasma_sphere_r", po::value<double>()->default_value(0), "radius of spherical plasma slab")
//        ("plasma_sphere_spacing", po::value<double>()->default_value(0), "plasma sphere spacing (center to center)" )
//        ("plasma_sphere_tight_packing", po::value<bool>()->default_value(true), "tight packing for plasma spheres (set false for square grid)" )
//
//        ("ppc", po::value<int>()->default_value(0), "number of particles per cell in plasma slab region (each species)")
//        ("ppc_sphere", po::value<int>()->default_value(0), "number of particles per cell in plasma slab region (each species) (spheres)")
//        ("ppc_el", po::value<int>()->default_value(0), "number of electron particles per cell in plasma slab region (overrides 'ppc')")
//        ("ppc_io", po::value<int>()->default_value(0), "number of ion particles per cell in plasma slab region (overrides 'ppc')")
//        ("ppc_el_sphere", po::value<int>()->default_value(0), "number of electron particles per cell in plasma slab region (overrides 'ppc') (spheres)")
//        ("ppc_io_sphere", po::value<int>()->default_value(0), "number of ion particles per cell in plasma slab region (overrides 'ppc') (spheres)")
//        ("vtherm_el", po::value<double>(), "electron thermal velocity")
//        ("vtherm_io", po::value<double>(), "ion thermal velocity")
//        ("n_ion_over_nc", po::value<double>(), "n_ion / nc for actual plasma density")
//
//        ("n_ramp_x_min", po::value<int>()->default_value(0), "density ramp width on x min")
//        ("n_ramp_x_max", po::value<int>()->default_value(0), "density ramp width on x max")
//        ("n_ramp_y_min", po::value<int>()->default_value(0), "density ramp width on y min")
//        ("n_ramp_y_max", po::value<int>()->default_value(0), "density ramp width on y max")
//        ("n_ramp_z_min", po::value<int>()->default_value(0), "density ramp width on z min")
//        ("n_ramp_z_max", po::value<int>()->default_value(0), "density ramp width on z max")
//
//        ("boundary_type_x_left",  po::value<int>(), "left x boundary type" )
//        ("boundary_type_x_right", po::value<int>(), "right x boundary type" )
//        ("boundary_type_y_left",  po::value<int>(), "left y boundary type" )
//        ("boundary_type_y_right", po::value<int>(), "right y boundary type" )
//        ("boundary_type_z_left",  po::value<int>(), "left z boundary type" )
//        ("boundary_type_z_right", po::value<int>(), "right z boundary type" )
//        ("damp_width", po::value<int>()->default_value(20), "damping region width in cells")

          ("B0", po::value<std::string>()->default_value("0,0,0"), "Initial B field (expression)")
          ("E0", po::value<std::string>()->default_value("0,0,0"), "Initial E field (expression)")

//        ("vdrift_el_x", po::value<double>()->default_value(0), "initial electron drift velocity in units of c, x component")
//        ("vdrift_el_y", po::value<double>()->default_value(0), "initial electron drift velocity in units of c, y component")
//        ("vdrift_el_z", po::value<double>()->default_value(0), "initial electron drift velocity in units of c, z component")
//        ("vdrift_io_x", po::value<double>()->default_value(0), "initial ion drift velocity in units of c, x component")
//        ("vdrift_io_y", po::value<double>()->default_value(0), "initial ion drift velocity in units of c, y component")
//        ("vdrift_io_z", po::value<double>()->default_value(0), "initial ion drift velocity in units of c, z component")
//        ("two_species_el", po::value<bool>()->default_value(false), "two electron species")
//        ("vdrift_el_x_2", po::value<double>()->default_value(0), "initial second species electron drift velocity in units of c, x component")
//        ("vdrift_el_y_2", po::value<double>()->default_value(0), "initial second species electron drift velocity in units of c, y component")
//        ("vdrift_el_z_2", po::value<double>()->default_value(0), "initial second species electron drift velocity in units of c, z component")
//        ("two_species_io", po::value<bool>()->default_value(false), "two ion species")
//        ("vdrift_io_x_2", po::value<double>()->default_value(0), "initial second species ion drift velocity in units of c, x component")
//        ("vdrift_io_y_2", po::value<double>()->default_value(0), "initial second species ion drift velocity in units of c, y component")
//        ("vdrift_io_z_2", po::value<double>()->default_value(0), "initial second species ion drift velocity in units of c, z component")
//        ("mix_ratio", po::value<int>()->default_value(1), "density ratio n1/n2 (integer)")
//        ("stationary_io", po::value<bool>()->default_value(false), "stationary ions")
//        ("heavy_particle_ratio", po::value<int>()->default_value(1), "heavy particle weight (integer)")
//
//        ("cells_per_wavelength", po::value<double>(), "cells per laser pulse wavelength (in appropriate direction, goes into dx, dy or dz=1/cpwl)")
//
//        ("pulse_type", po::value<int>()->default_value(0), "laser pulse type")
//        ("pulse_axis", po::value<int>()->default_value(1), "laser pulse axis 1 - X, 2 - Y, 3 - Z")
//        ("pulse_length", po::value<int>(), "laser pulse length (cells)")
//        ("pulse_intensity", po::value<double>(), "laser pulse intensity a0")
//        ("pulse_polarization", po::value<int>()->default_value(0), "laser pulse polarization")
//        ("pulse_width", po::value<double>(), "Gaussian pulse waist half-width W0 (cells)")
//        ("pulse_focus", po::value<double>(), "Gaussian pulse focal point x-coordinate (cells)")
//
//        ("diag_fields", po::value<bool>()->default_value(false), "cell fields diagnostics")
//        ("diag_npart", po::value<bool>()->default_value(false), "cell particle count diagnostics")
//        ("diag_particles", po::value<bool>()->default_value(false), "global particle diagnostics")
//        ("diag_trajectories", po::value<bool>()->default_value(false), "particle trajectories diagnostics (do not use)")
//        ("diag_energy", po::value<bool>()->default_value(false), "energy diagnostics")
//        ("out_start", po::value<int>()->default_value(0), "equidistant output start (fields, npart, particles)")
//        ("out_stop", po::value<int>()->default_value(0), "equidistant output stop (fields, npart, particles)")
//        ("out_each", po::value<int>()->default_value(1), "equidistant output step (fields, npart, particles)")
//        ("out_energy_each", po::value<int>()->default_value(0), "energy output step")
//        ("out_slices_each", po::value<int>()->default_value(0), "energy output step")
//
//        ("out_steps", po::value< std::string >()->default_value(" "), "list of discrete diagnostics steps (fields, npart, particles)")
//
//        ("out_slices_x", po::value< std::string >()->default_value(" "), "list of x-coordinate of y-z plane output slices")
//        ("out_slices_y", po::value< std::string >()->default_value(" "), "list of y-coordinate of x-z plane output slices")
//        ("out_slices_z", po::value< std::string >()->default_value(" "), "list of z-coordinate of x-y plane output slices")

        #ifdef MPI_PARALLEL
        ("nproc_x", po::value<int>()->default_value(0), "number of MPI processes in x-direction")
        ("nproc_y", po::value<int>()->default_value(0), "number of MPI processes in y-direction")
        ("nproc_z", po::value<int>()->default_value(0), "number of MPI processes in z-direction")
        #endif

    ;

    po::variables_map vm;

    std::ifstream ifs(filename.c_str());
    if (!ifs){
        LE << "can not open config file: " << filename << "\n";
        exit(30);
    }else{
        po::store(po::parse_config_file(ifs, generic), vm);
        po::notify(vm);
    }

    //TODO: Move all logic for default values in Settings.
    //      Call functions from Settings like settings->set_nsteps(vm["nsteps"].as<int>());
    //      Settings will have to handle the logic at the end, when all variables are set!

    _settings->nsteps = vm["nsteps"].as<int>();
    parse_string(vm["phys_size"].as< std::string >(), _settings->phys_size);

//    settings_->dy_factor_ = vm["dy_factor"].as<double>();
//    settings_->dz_factor_ = vm["dz_factor"].as<double>();
//    settings_->dt_factor_ = vm["dt_factor"].as<double>();

    #ifdef MOVING_WINDOW
    settings_->window_size_ = vm["window_size"].as<int>();
    settings_->window_initial_size_ = (vm["window_initial_size"].as<int>() ? vm["window_initial_size"].as<int>() : settings_->window_size_) ;
    settings_->window_start_ = vm["window_start"].as<int>();

    settings_->window_stop_ = (vm["window_stop"].as<int>() ? vm["window_stop"].as<int>() : settings_->nsteps_);
    #endif

    parse_string(vm["plasma_min"].as< std::string >(), _settings->domain.plasma.min);
    parse_string(vm["plasma_max"].as< std::string >(), _settings->domain.plasma.max);


//
//    settings_->nspheres_ = vm["nspheres"].as<int>();
//    settings_->plasma_sphere_centre_ = V3D(vm["plasma_sphere_x"].as<double>(),
//                                         vm["plasma_sphere_y"].as<double>(),
//                                         vm["plasma_sphere_z"].as<double>());
//    settings_->plasma_sphere_radius_ = vm["plasma_sphere_r"].as<double>();
//    settings_->plasma_sphere_spacing_ = vm["plasma_sphere_spacing"].as<double>()
//                                      ? vm["plasma_sphere_spacing"].as<double>()
//                                      : 2* settings_->plasma_sphere_radius_; // default sphere spacing = 2*R
//    settings_->plasma_sphere_tight_packing_ = vm["plasma_sphere_tight_packing"].as<bool>();
//
//    settings_->vtherm_el_ = vm["vtherm_el"].as<double>();
//    settings_->vtherm_io_ = vm["vtherm_io"].as<double>();
//
//    settings_->n_ion_over_nc_ = vm["n_ion_over_nc"].as<double>();
//
//    settings_->ppc_el_ = vm["ppc_el"].as<int>() ? vm["ppc_el"].as<int>() : vm["ppc"].as<int>();
//    settings_->ppc_io_ = vm["ppc_io"].as<int>() ? vm["ppc_io"].as<int>() : vm["ppc"].as<int>();
//
//    if (vm["ppc_sphere"].as<int>() || vm["ppc_el_sphere"].as<int>() || vm["ppc_io_sphere"].as<int>()){
//        settings_->ppc_el_sphere_ = vm["ppc_el_sphere"].as<int>() ? vm["ppc_el_sphere"].as<int>() : vm["ppc_sphere"].as<int>();
//        settings_->ppc_io_sphere_ = vm["ppc_io_sphere"].as<int>() ? vm["ppc_io_sphere"].as<int>() : vm["ppc_sphere"].as<int>();
//    }else{
//        settings_->ppc_el_sphere_ = settings_->ppc_el_;
//        settings_->ppc_io_sphere_ = settings_->ppc_io_;
//    }
//
//    settings_->bx_min_t_ = get_border_type(vm["boundary_type_x_left"].as<int>());
//    settings_->bx_max_t_ = get_border_type(vm["boundary_type_x_right"].as<int>());
//    settings_->by_min_t_ = get_border_type(vm["boundary_type_y_left"].as<int>());
//    settings_->by_max_t_ = get_border_type(vm["boundary_type_y_right"].as<int>());
//    settings_->bz_min_t_ = get_border_type(vm["boundary_type_z_left"].as<int>());
//    settings_->bz_max_t_ = get_border_type(vm["boundary_type_z_right"].as<int>());
//
//    settings_->n_ramp_x_min_ = vm["n_ramp_x_min"].as<int>();
//    settings_->n_ramp_x_max_ = vm["n_ramp_x_max"].as<int>();
//    settings_->n_ramp_y_min_ = vm["n_ramp_y_min"].as<int>();
//    settings_->n_ramp_y_max_ = vm["n_ramp_y_max"].as<int>();
//    settings_->n_ramp_z_min_ = vm["n_ramp_z_min"].as<int>();
//    settings_->n_ramp_z_max_ = vm["n_ramp_z_max"].as<int>();
//
//    settings_->damp_width_ = vm["damp_width"].as<int>();

    _settings->domain.fields._B0 = vm["B0"].as< std::string >();
    _settings->domain.fields._E0 = vm["E0"].as< std::string >();

//    settings_->vdrift_el_.x() = vm["vdrift_el_x"].as<double>();
//    settings_->vdrift_el_.y() = vm["vdrift_el_y"].as<double>();
//    settings_->vdrift_el_.z() = vm["vdrift_el_z"].as<double>();
//    settings_->vdrift_io_.x() = vm["vdrift_io_x"].as<double>();
//    settings_->vdrift_io_.y() = vm["vdrift_io_y"].as<double>();
//    settings_->vdrift_io_.z() = vm["vdrift_io_z"].as<double>();
//
//    if(vm["two_species_el"].as<bool>()){
//        settings_->vdrift_el_2_.x() = vm["vdrift_el_x_2"].as<double>();
//        settings_->vdrift_el_2_.y() = vm["vdrift_el_y_2"].as<double>();
//        settings_->vdrift_el_2_.z() = vm["vdrift_el_z_2"].as<double>();
//    }else{ settings_->vdrift_el_2_ = settings_->vdrift_el_; }
//    if(vm["two_species_io"].as<bool>()){
//        settings_->vdrift_io_2_.x() = vm["vdrift_io_x_2"].as<double>();
//        settings_->vdrift_io_2_.y() = vm["vdrift_io_y_2"].as<double>();
//        settings_->vdrift_io_2_.z() = vm["vdrift_io_z_2"].as<double>();
//    }else{ settings_->vdrift_io_2_ = settings_->vdrift_io_; }
//    settings_->mix_ratio_ = vm["mix_ratio"].as<int>();
//    settings_->heavy_particle_ratio_ = vm["heavy_particle_ratio"].as<int>();
//
//    settings_->stationary_io_ = vm["stationary_io"].as<bool>();
//
//    settings_->cells_per_wavelength_   = vm["cells_per_wavelength"].as<double>();
//
//    settings_->pulse_type_         = get_pulse_type(vm["pulse_type"].as<int>());
//    settings_->pulse_axis_         = get_axis(vm["pulse_axis"].as<int>());
//    settings_->pulse_length_       = vm["pulse_length"].as<int>();
//    settings_->pulse_intensity_    = vm["pulse_intensity"].as<double>();
//    settings_->pulse_polarization_ = get_pulse_polarization(vm["pulse_polarization"].as<int>());
//    settings_->pulse_width_        = vm["pulse_width"].as<double>();
//    settings_->pulse_focus_        = vm["pulse_focus"].as<double>();
//
//    settings_->diag_fields_ = vm["diag_fields"].as<bool>();
//    settings_->diag_npart_ = vm["diag_npart"].as<bool>();
//    settings_->diag_particles_ = vm["diag_particles"].as<bool>();
//    settings_->diag_trajectories_ = vm["diag_trajectories"].as<bool>();
//    settings_->diag_energy_ = vm["diag_energy"].as<bool>();
//
//    settings_->out_start_ = vm["out_start"].as<int>();
//    settings_->out_stop_ = vm["out_stop"].as<int>() ? vm["out_stop"].as<int>() : settings_->nsteps_;
//    settings_->out_each_ = vm["out_each"].as<int>();
//    settings_->out_energy_each_ = vm["out_energy_each"].as<int>();
//    settings_->out_slices_each_ = vm["out_slices_each"].as<int>();
//
//    std::string tmpstr;
//
//    tmpstr = vm["out_steps"].as< std::string >(); parse_string(tmpstr, settings_->out_steps_);
//
//    tmpstr = vm["out_slices_x"].as< std::string >(); parse_string(tmpstr, settings_->out_slices_x_);
//    tmpstr = vm["out_slices_y"].as< std::string >(); parse_string(tmpstr, settings_->out_slices_y_);
//    tmpstr = vm["out_slices_z"].as< std::string >(); parse_string(tmpstr, settings_->out_slices_z_);

    #ifdef MPI_PARALLEL
    settings_->nproc_x_ = vm["nproc_x"].as<int>();
    settings_->nproc_y_ = vm["nproc_y"].as<int>();
    settings_->nproc_z_ = vm["nproc_z"].as<int>();
    #endif

}

void InputReaderBoost::parse_string(const std::string &s, std::vector<int> &elems) {
    std::stringstream ss(s);
    std::string item;
    elems.clear();
    while(std::getline(ss, item, ',')) {
        int e;
        sscanf(item.c_str(),"%i",&e);
        elems.push_back(e);
    }
}

//TODO: Rewrite in templates
void InputReaderBoost::parse_string(const std::string &s, VI &vect) {
    std::stringstream ss(s);
    std::string item;
    std::vector<int> elems;
    while(std::getline(ss, item, ',')) {
        int e;
        sscanf(item.c_str(),"%i",&e);
        elems.push_back(e);
    }
    if(elems.size() == 3){ vect = VI(elems[0],elems[1],elems[2]); }
    else{ LE << "cannot parse VI"; exit(31); }
}

//TODO: Rewrite in templates
void InputReaderBoost::parse_string(const std::string &s, VFLD &vect) {
    std::stringstream ss(s);
    std::string item;
    std::vector<double> elems;
    while(std::getline(ss, item, ',')) {
        double e;
        sscanf(item.c_str(),"%lf",&e);
        elems.push_back(e);
    }
    if(elems.size() == 3){ vect = VFLD(elems[0],elems[1],elems[2]); }
    else{ LE << "cannot parse VD"; exit(32); }
}

}
}
