/*
 * domain.cpp
 *
 *  Created on: 18 May 2016
 *      Author: strazce
 */

#include "data/domain.h"
#include "utils/basic_logger.h"
#include <cmath>


namespace opice {
namespace data {

using conf::DomainSettings;

Domain::Domain(DomainSettings* settings):
    _settings(settings),
    _grid(new Grid(_settings->size))
    {
}

void Domain::init() {
    init_plasma();
    init_fields();
}

void Domain::init_plasma() {
    LI << "Loading particles... " << _settings->plasma.min << " -> " << _settings->plasma.max;
}

void Domain::init_fields() {
    LI << "Loading fields...";

    for(int i = 0; i<_grid->size().x; ++i){
        for(int j = 0; j<_grid->size().y; ++j){
            for(int k = 0; k<_grid->size().z; ++k){
                _grid->B[VI(i,j,k)] = _settings->fields.B0->get(VPOS(i,j,k), 0);
            }
        }
    }

    for(int i = 0; i<_grid->size().x; ++i){
        for(int j = 0; j<_grid->size().y; ++j){
            for(int k = 0; k<_grid->size().z; ++k){
                _grid->E[VI(i,j,k)] = _settings->fields.E0->get(VPOS(i,j,k), 0);
            }
        }
    }
}

} /* namespace data */
} /* namespace opice */
