/*
 * experiment.cpp
 *
 *  Created on: 18 May 2016
 *      Author: strazce
 */

#include "experiment.h"

#include "utils/basic_logger.h"

#include "data/box.h"

#include "data/grid.h"

#include "physics/field_fdtd1.h"

#include <iostream>

#include "conf/settings.h"

namespace opice {

using data::Box;
using data::Grid;
using physics::FieldFDTD1;
using conf::Settings;

Experiment::Experiment(std::unique_ptr<Settings> settings)
    : _settings(std::move(settings)),
      _box(new Box(_settings.get()))
      {
}

void Experiment::load_data(){
    _box->init_domain();
    LI << "Grid size:" << _box->domain()->grid()->size();
}

void Experiment::run(){

    Grid* grid = _box->domain()->grid();



    VSTI s(0.01, 0.1, 0.1, 0.1);
    std::unique_ptr<FieldFDTD1> field_solver(new FieldFDTD1(s, grid));

    LI << "Initial conditions:";
    for(int i=0; i<_settings->domain.size.x; ++i){
        VI idx = VI(i,5,5);
        std::cout << grid->E[idx] << "\t";
    }
    std::cout << std::endl;

    LI << "Starting simulation:";
    for(int n=0; n < _settings->nsteps; ++n){
        field_solver->advance();
    }


    LI << "Results:";
    for(int i=0; i<_settings->domain.size.x; ++i){
        VI idx = VI(i,5,5);
        std::cout << grid->E[idx] << "\t";
    }
    std::cout << std::endl;
}

Experiment::~Experiment() {
    // TODO Auto-generated destructor stub
}

} /* namespace opice */
