/*
 * common.h
 *
 */

#ifndef INCLUDE_COMMON_H_
#define INCLUDE_COMMON_H_

//TODO: Get rid of this #define, move interpolation order selection to a template
#define INTERPOLATION_ORDER 1
//TODO: This would be defined by the configure script
#define DIM_POS 3
#define DIM_VEL 3
//#define INTRUSIVE_LIST


#ifndef BOOST_SYSTEM_NO_DEPRECATED
#define BOOST_SYSTEM_NO_DEPRECATED 1
#endif

#include <cmath>
#include <vector>
#include "data/vector.h"
#include "data/container_soa.h"

namespace opice {

using Scalar = double;
template<int N> using VIDX = data::vector::Vector<int,N,true>;

using VFLD = data::vector::Vector<Scalar,DIM_VEL,true>;   ///< Vector representing a vector field element (those which would be 3D in a 2D3V version)
using VPOS = data::vector::Vector<Scalar,DIM_POS,true>;   ///< Vector representing a position of a particle (or what would be 2D in a 2D3V version)
using VSTI = data::vector::Vector<Scalar,DIM_POS+1,true>; ///< Vector representing a space-time interval element (spatial dimensions + 1)

using VectorField = data::container_soa::ContainerSoa<VFLD,DIM_POS>; ///< Container representing a vector field
using ScalarField = data::container_soa::ContainerSoa<Scalar,DIM_POS>; ///< Container representing a scalar field

using VI = VIDX<DIM_POS>;


template<typename T> inline T sqr(T x){ return x*x; }
template<typename T> inline T cub(T x){ return x*x*x; }
template<typename T> inline T cross(T x, T y) { return x.cross(y); }

// Return a mod b (even in case a is negative)
template<typename T> T mod(const T& a, const T& b){ return (a % b + b) % b; }


#ifdef M_PIl
const double PI = M_PIl;
#else
const double PI = 3.1415926535897932384626;
#endif

const double isqrt3 =  1/sqrt(3);
const double isqrt2 =  1/sqrt(2);

enum Axis { A_X = 0, A_Y = 1, A_Z = 2, A_INVALID = -666 };
enum Direction { LEFT = -1, RIGHT = 1, CENTER = 0, DIR_INVALID = -666 };

inline int zero_based_left_right(Direction direction) { return (static_cast<int>(direction)+1)/2; }
inline int zero_based_axis_number(Axis axis) { return static_cast<int>(axis); }

enum BorderType {
    BD_INVALID,
    BD_PERIODIC,
    BD_REFLECTIVE,
    BD_ABSORBING,
    BD_ABSORBING_THERM,
    BD_WINDOW,
    BD_COMM
};


}


#endif /* INCLUDE_COMMON_H_ */
