/*
 * config_expression.h
 *
 *  Created on: Jun 15, 2016
 *      Author: strazce
 */

#ifndef INCLUDE_CONF_VECTOR_CONFIG_EXPRESSION_H_
#define INCLUDE_CONF_VECTOR_CONFIG_EXPRESSION_H_

#include "conf/scalar_config_expression.h"
#include "common.h"
#include "utils/basic_logger.h"

#include <vector>
#include <memory>
#include <string>
#include <boost/tokenizer.hpp>

namespace opice {
namespace conf {

/*** @todo  This has to be put somewhere else. The tokenizer, etc should absolutely NOT be in
 *          the header file. The header is included in settings.h, which are included
 *          in everything else! We have to keep this dependency at the absolute minimum.
 */

template<std::size_t DIM>
static auto parse_vector_config_expression(const std::string &expr_string){

    std::vector<std::unique_ptr<ScalarConfigExpression>> subexpressions;

    using vector_tokenizer = boost::tokenizer<boost::escaped_list_separator<char>>;

    vector_tokenizer tok(expr_string, boost::escaped_list_separator<char>('\\', ',', '\"'));
    for(auto it: tok) {
        subexpressions.push_back( std::make_unique<ScalarConfigExpression>(it) );
    }

    if (subexpressions.size() != DIM){
        LE << "Incorrect number of dimensions in expression: " << expr_string;
        LE << "  expected: " << DIM << ", parsed: "  << subexpressions.size();
        LE << "Tokenizer built: ";
        for(unsigned int i=0; i<subexpressions.size(); ++i){
            LE << "  " << i << ": " << subexpressions[i]->original_expr();
        }
    }

    return subexpressions;
}

template<std::size_t DIM>
class VectorConfigExpression {

public:
    VectorConfigExpression(const std::string &expr_string)
        : _scalars(std::move(parse_vector_config_expression<DIM>(expr_string)))
        {}

    auto get(const VPOS &r, Scalar t) {
        //FIXME: 3D only!
        //TODO: Templatize with integer sequence
        return data::vector::Vector<Scalar, DIM, true>( //FIXME: Always returning a SoA-ready vector, can we get rid of the SoA/non-SoA notion here?
                _scalars[0]->get(r,t),
                _scalars[1]->get(r,t),
                _scalars[2]->get(r,t)
                );
    }

private:
    std::vector<std::unique_ptr<ScalarConfigExpression>> _scalars;

};

}
}



#endif /* INCLUDE_CONF_SCALAR_CONFIG_EXPRESSION_H_ */
