/*
 * vector_base.h
 *
 *  Created on: 19 May 2016
 *      Author: strazce
 */

#ifndef INCLUDE_DATA_VECTOR_BASE_H_
#define INCLUDE_DATA_VECTOR_BASE_H_

#include "vector_details.h"

#include <cstddef>

namespace opice {
namespace data {
namespace vector {

/*** @brief Base class contains functions which are have the same
 *          implementation for all vector variants, and provides
 *          the CRTP dispatching interface for the others
 * */
template<typename T, typename Inner>
struct VectorBase
    : Details<Inner>
{
    using value = typename Details<Inner>::value;
    using ref   = typename Details<Inner>::ref;
    using cref  = typename Details<Inner>::cref;

    ref  operator[](std::size_t k){
        return static_cast<T*>(this)->ret_op(k);
    }
    cref operator[](std::size_t k) const {
        return static_cast<const T*>(this)->ret_op(k);
    }
    //TODO: also provide begin/end iterators
    constexpr auto size(){
        return static_cast<const T*>(this)->vector_dimension;
    }

};

}
}
}


#endif /* INCLUDE_DATA_VECTOR_BASE_H_ */
