/*
 * vector_expressions.h
 *
 *  Created on: Apr 7, 2016
 *      Author: strazce
 */

#ifndef INCLUDE_DATA_VECTOR_EXPRESSIONS_H_
#define INCLUDE_DATA_VECTOR_EXPRESSIONS_H_

#include <cstdlib>
#include "vector_details.h"

namespace opice {
namespace data {
namespace expressions {

template<typename T>
using Details = vector::Details<T>;

template<typename T, typename Inner, std::size_t DIM>
struct Expressions {

    using value = typename Details<Inner>::value;
    using ref   = typename Details<Inner>::ref;
    using cref  = typename Details<Inner>::cref;

    constexpr auto size() {
        return DIM;
    }

    template<typename V> constexpr
    T& operator+=(V&& b) {
        for(std::size_t i=0; i<DIM; ++i){
            (*static_cast<T*>(this))[i] += b[i];
        }
        return static_cast<T&>(*this);
    }

    template<typename V> constexpr
    T& operator-=(V&& b) {
        for(std::size_t i=0; i<DIM; ++i){
            (*static_cast<T*>(this))[i] -= b[i];
        }
        return static_cast<T&>(*this);
    }

    template<typename R, typename std::enable_if_t<std::is_arithmetic<R>::value,
            R>* = nullptr>
    constexpr
    T& operator*=(R c) {
        for(std::size_t i=0; i<DIM; ++i){
            (*static_cast<T*>(this))[i] *= c;
        }
        return static_cast<T&>(*this);
    }

    template<typename R, typename std::enable_if_t<std::is_arithmetic<R>::value,
            R>* = nullptr>
    constexpr
    T& operator/=(R c) {
        for(std::size_t i=0; i<DIM; ++i){
            (*static_cast<T*>(this))[i] /= c;
        }
        return static_cast<T&>(*this);
    }

};

}}} // namespace opice::data::expressions

#endif /* INCLUDE_DATA_VECTOR_EXPRESSIONS_H_ */
