/*
 * vector_impl_1d.h
 *
 *  Created on: Apr 7, 2016
 *      Author: strazce
 */

#ifndef INCLUDE_DATA_VECTOR_IMPL_1D_H_
#define INCLUDE_DATA_VECTOR_IMPL_1D_H_

#include <cmath>
#include <algorithm>
#include <type_traits>
#include <cstddef>
#include <ostream>
#include <assert.h>

namespace opice {
namespace data {
namespace vector {

template<typename T, bool SOA_READY>
struct alignas(ALIGN) Vector<T, 1, SOA_READY>
    : VectorBase<Vector<T, 1, SOA_READY>, T>,
      Expr<Vector<T, 1, SOA_READY>, T, 1>
{
    static constexpr unsigned int DIM = 1;
    static constexpr unsigned int SOA_VDIM = 1;

    using value = typename Details<T>::value;
    using ref = typename Details<T>::ref;
    using cref = typename Details<T>::cref;
    using VAL = Vector<value, 1, true>;
    using CREF = Vector<cref, 1, true>;
    using REF = Vector<ref, 1, true>;

    T x;

    constexpr Vector() = delete;
    constexpr Vector(T ix) :
            x(ix) {
    }
    template<typename T2, bool WHATEVER>
    constexpr Vector(Vector<T2, DIM, WHATEVER> v) :
            x(v.x) {
    }
    template<typename T2, bool WHATEVER>
    Vector& operator=(Vector<T2, DIM, WHATEVER> && v)  {
        x = v.x;
        return *this;
    }
    constexpr Vector(Vector const & v) :
            x(v.x) {
    }

    cref operator[](unsigned int k) const {
        assert(k==0); (void)k; //HACK: (void)k; silences the "unused parameter" warning
        return x;
    }

    ref operator[](unsigned int k) {
        assert(k==0); (void)k; //HACK: (void)k; silences the "unused parameter" warning
        return x;
    }

    template<typename T2, bool SOA2>
    constexpr bool operator==(const Vector<T2,1,SOA2>& b){
        return x == b.x;
    }

    static constexpr auto inline Zeros() { return Vector(0); }
    static constexpr auto inline Ones()  { return Vector(1); }

};


}}} // namespace opice::data::vector



#endif /* INCLUDE_DATA_VECTOR_IMPL_1D_H_ */
