/*
 * vector_impl_2d.h
 *
 *  Created on: Apr 7, 2016
 *      Author: strazce
 */

#ifndef INCLUDE_DATA_VECTOR_IMPL_2D_H_
#define INCLUDE_DATA_VECTOR_IMPL_2D_H_

#include <cmath>
#include <algorithm>
#include <type_traits>
#include <cstddef>
#include <ostream>

#include "vector_details.h"
#include "vector_expressions.h"

namespace opice {
namespace data {
namespace vector {

template<typename T>
struct alignas(ALIGN) Vector<T, 2, false>
    : VectorBase<Vector<T, 2, false>, T>,
      Expr<Vector<T, 2, false>, T, 2>
{
    static constexpr unsigned int DIM = 2;

    using value = typename Details<T>::value;
    using ref = typename Details<T>::ref;
    using cref = typename Details<T>::cref;
    using VAL = Vector<value, 2, false>;
    using CREF = Vector<cref, 2, false>;
    using REF = Vector<ref, 2, false>;

    T x, y;

    constexpr Vector() = delete;
    constexpr Vector(T ix, T iy) :
            x(ix), y(iy) {
    }
    template<typename T2, bool SOA_READY>
    constexpr Vector(Vector<T2, DIM, SOA_READY> v) :
            x(v.x), y(v.y) {
    }
    template<typename T2, bool SOA_READY>
    Vector& operator=(Vector<T2, DIM, SOA_READY> && v) {
        x = v.x;
        y = v.y;
        return *this;
    }
    constexpr Vector(Vector const & v) :
            x(v.x), y(v.y) {
    }

    /// @internal Pointer arithmetics validity must be checked compile-time
    cref operator[](unsigned int k) const {
        return *(&x + k);
    }

    ref operator[](unsigned int k) {
        return *(&x + k);
    }

    template<typename T2, bool SOA2>
    constexpr bool operator==(const Vector<T2,2,SOA2>& b){
        return x == b.x && y == b.y;
    }

    static constexpr Vector Zeros() { return Vector(0,0); }
    static constexpr Vector Ones()  { return Vector(1,1); }

};

template<typename T>
struct alignas(ALIGN) Vector<T, 2, true>
    : VectorBase<Vector<T, 3, true>, T>,
      Expr<Vector<T, 2, true>, T, 2>
{
    static constexpr unsigned int DIM = 2;
    static constexpr unsigned int SOA_VDIM = 2;

    using value = typename Details<T>::value;
    using ref = typename Details<T>::ref;
    using cref = typename Details<T>::cref;
    using VAL = Vector<value, 2, true>;
    using CREF = Vector<cref, 2, true>;
    using REF = Vector<ref, 2, true>;

    T x, y;

    constexpr Vector() = delete;

    constexpr Vector(T ix, T iy) :
            x(ix), y(iy) {
    }

    template<typename T2, bool SOA_READY>
    constexpr Vector(Vector<T2, DIM, SOA_READY> v) :
            x(v.x), y(v.y) {
    }

    constexpr Vector(Vector const & v) :
            x(v.x), y(v.y) {
    }

    template<typename T2, bool SOA_READY>
    Vector& operator=(Vector<T2, DIM, SOA_READY> v) {
        x = v.x;
        y = v.y;
        return *this;
    }

    template<typename T2, bool SOA_READY>
    Vector& operator=(const Vector<T2&, DIM, SOA_READY> & v) {
        x = v.x;
        y = v.y;
        return *this;
    }

    cref operator[](unsigned int k) const {
        return (k == 0) ? x : y;
    }

    ref operator[](unsigned int k) {
        return (k == 0) ? x : y;
    }

    template<typename T2, bool SOA2>
    constexpr bool operator==(const Vector<T2,2,SOA2>& b){
        return x == b.x && y == b.y;
    }

    static constexpr Vector Zeros() { return Vector(0,0); }
    static constexpr Vector Ones()  { return Vector(1,1); }

};

/// @internal Compile-time checks for pointer arithmetics validity
//TODO: Generalize the assertions, or move them to some more sensible place
using NonSoaV2f = Vector<float, 2, false>;
using NonSoaV2d = Vector<double, 2, false>;
static_assert(offsetof(NonSoaV2f,y) == sizeof(float), "Offset of y is not at float size");
static_assert(offsetof(NonSoaV2d,y) == sizeof(double), "Offset of y is not at double size");

}}} // namespace opice::data::vector



#endif /* INCLUDE_DATA_VECTOR_IMPL_2D_H_ */
