/*
 * field_fdtd1.cpp
 *
 */

#include "physics/field_fdtd1.h"
#include <iostream>

namespace opice {
namespace physics {

FieldFDTD1::FieldFDTD1(const VSTI& s, data::Grid* g)
    : FieldSolver(s),
      //,GridIterator<FieldFDTD1>()
      _grid(g),
      _fB0(VFLD::Zeros())
{
}

void FieldFDTD1::advance(){

/*    VI phys_min = grid_->boundary->phys_min_;
    VI phys_max = grid_->boundary->phys_max_; */

    /*
    // B, half-step:                    B(t) --> B(t+1/2)
    // B(t+dt/2) pouziva B(t) a E(t) v i+1, j+1, k+1
    cell_apply(*this, phys_min, phys_max, &FieldFDTD1::it_advance_B_half);
    grid_->boundary->set_B();

    // E:                               E(t) --> E(t+dt)
    // E(t+dt) pouziva E(t) a B(t+dt/2) v i-1, j-1, k-1
    cell_apply(*this, phys_min, phys_max,
            &FieldFDTD1::it_advance_E);
    grid_->boundary->set_E();

    // B, half-step:                    B(t+dt/2) --> B(t+dt)
    // B(t) pouziva B(t-dt/2) a E(t) v i+1, j+1, k+1
    cell_apply(*this, phys_min, phys_max, &FieldFDTD1::it_advance_B_half);
    grid_->boundary->set_B();
*/

    for(int i=0; i<_grid->size().x-1; ++i){
        for(int j=0; j<_grid->size().y-1; ++j){
            for(int k=0; k<_grid->size().z-1; ++k){
                advance_B_half(VI(i,j,k));
            }
        }
    }
    for(int i=1; i<_grid->size().x; ++i){
        for(int j=1; j<_grid->size().y; ++j){
            for(int k=1; k<_grid->size().z; ++k){
                advance_E(VI(i,j,k));
            }
        }
    }
    for(int i=0; i<_grid->size().x-1; ++i){
        for(int j=0; j<_grid->size().y-1; ++j){
            for(int k=0; k<_grid->size().z-1; ++k){
                advance_B_half(VI(i,j,k));
            }
        }
    }

}

inline void FieldFDTD1::advance_B_half(const VI &idx){
    _grid->B[idx] += 0.5 * _dt * rot(_grid->E, idx, _idr, 1);
}

inline void FieldFDTD1::advance_E(const VI &idx){
    _grid->E[idx] += 2*PI*_dt*_grid->J[idx] + _dt * rot(_grid->B, idx, _idr, -1);
}

#if DIM_POS == 2

// Half-step B.
inline VFLD FieldFDTD1::advance_B_half(const VI &idx, const VI &d_idx){
    VFLD B;

    const int i=idx[0];    const int  j=idx[1];
    const int ii=d_idx[0]; const int jj=d_idx[1];

    B.x() = _grid->B(idx).x() + 0.5*_dt
            * ( (_grid->E[i][j].z()  - _grid->E[i][jj].z())*_idy );
    B.y() = _grid->B(idx).y() + 0.5*_dt
            * ( (_grid->E[ii][j].z() - _grid->E[i][j].z())*_idx );
    B.z() = _grid->B(idx).z() + 0.5*_dt
            * ( (_grid->E[i][jj].x() - _grid->E[i][j].x())*_idy
              - (_grid->E[ii][j].y() - _grid->E[i][j].y())*_idx );
    return B;
}

inline VFLD FieldFDTD1::advance_E(const VI &idx, const VI &d_idx){
    VFLD E;

    const int i=idx[0];    const int  j=idx[1];
    const int ii=d_idx[0]; const int jj=d_idx[1];

    E.x() = _grid->E(idx).x() - 2*PI*_dt*_grid->J(idx).x() + _dt
            * ( (_grid->B[i][j].z()  - _grid->B[i][jj].z())*_idy );
    E.y() = _grid->E(idx).y() - 2*PI*_dt*_grid->J(idx).y() + _dt
            * ( (_grid->B[ii][j].z() - _grid->B[i][j].z())*_idx );
    E.z() = _grid->E(idx).z() - 2*PI*_dt*_grid->J(idx).z() + _dt
            * ( (_grid->B[i][j].y() - _grid->B[ii][j].y())*_idx
             -  (_grid->B[i][j].x() - _grid->B[i][jj].x())*_idy );
    return E;
}

#endif








/*
#if DIM_POS == 3

// Half-step B.
inline VF FieldFDTD1::advance_B_half(const VI &idx, const VI &d_idx){
    VF B;

    const int i=idx[0]; const int j=idx[1]; const int k=idx[2];
    const int ii=d_idx[0]; const int jj=d_idx[1]; const int kk=d_idx[2];

    B.x() = _grid->B(idx).x() + 0.5*_dt
            * ( (_grid->E[i][j][kk].y() - _grid->E[i][j][k].y())*_idz
              - (_grid->E[i][jj][k].z() - _grid->E[i][j][k].z())*_idy );
    B.y() = _grid->B(idx).y() + 0.5*_dt
            * ( (_grid->E[ii][j][k].z() - _grid->E[i][j][k].z())*_idx
              - (_grid->E[i][j][kk].x() - _grid->E[i][j][k].x())*_idz );
    B.z() = _grid->B(idx).z() + 0.5*_dt
            * ( (_grid->E[i][jj][k].x() - _grid->E[i][j][k].x())*_idy
              - (_grid->E[ii][j][k].y() - _grid->E[i][j][k].y())*_idx );
    return B;
}

inline VF FieldFDTD1::advance_E(const VI &idx, const VI &d_idx){
    VF E;

    const int i=idx[0]; const int j=idx[1]; const int k=idx[2];
    const int ii=d_idx[0]; const int jj=d_idx[1]; const int kk=d_idx[2];

    E.x() = _grid->E[i][j][k].x() - 2*PI*_dt*_grid->J[i][j][k].x() + _dt
            * ( (_grid->B[i][j][k].z() - _grid->B[i][jj][k].z())*_idy
             -  (_grid->B[i][j][k].y() - _grid->B[i][j][kk].y())*_idz );
    E.y() = _grid->E[i][j][k].y() - 2*PI*_dt*_grid->J[i][j][k].y() + _dt
            * ( (_grid->B[i][j][k].x() - _grid->B[i][j][kk].x())*_idz
             -  (_grid->B[i][j][k].z() - _grid->B[ii][j][k].z())*_idx );
    E.z() = _grid->E[i][j][k].z() - 2*PI*_dt*_grid->J[i][j][k].z() + _dt
            * ( (_grid->B[i][j][k].y() - _grid->B[ii][j][k].y())*_idx
             -  (_grid->B[i][j][k].x() - _grid->B[i][jj][k].x())*_idy );
    return E;
}

#elif DIM_POS == 2

// Half-step B.
inline VF FieldFDTD1::advance_B_half(const VI &idx, const VI &d_idx){
    VF B;

    const int i=idx[0];    const int  j=idx[1];
    const int ii=d_idx[0]; const int jj=d_idx[1];

    B.x() = _grid->B(idx).x() + 0.5*_dt
            * ( (_grid->E[i][j].z()  - _grid->E[i][jj].z())*_idy );
    B.y() = _grid->B(idx).y() + 0.5*_dt
            * ( (_grid->E[ii][j].z() - _grid->E[i][j].z())*_idx );
    B.z() = _grid->B(idx).z() + 0.5*_dt
            * ( (_grid->E[i][jj].x() - _grid->E[i][j].x())*_idy
              - (_grid->E[ii][j].y() - _grid->E[i][j].y())*_idx );
    return B;
}

inline VF FieldFDTD1::advance_E(const VI &idx, const VI &d_idx){
    VF E;

    const int i=idx[0];    const int  j=idx[1];
    const int ii=d_idx[0]; const int jj=d_idx[1];

    E.x() = _grid->E(idx).x() - 2*PI*_dt*_grid->J(idx).x() + _dt
            * ( (_grid->B[i][j].z()  - _grid->B[i][jj].z())*_idy );
    E.y() = _grid->E(idx).y() - 2*PI*_dt*_grid->J(idx).y() + _dt
            * ( (_grid->B[ii][j].z() - _grid->B[i][j].z())*_idx );
    E.z() = _grid->E(idx).z() - 2*PI*_dt*_grid->J(idx).z() + _dt
            * ( (_grid->B[i][j].y() - _grid->B[ii][j].y())*_idx
             -  (_grid->B[i][j].x() - _grid->B[i][jj].x())*_idy );
    return E;
}

#endif
*/




}
}
