/*
 * vector3d.cpp
 *
 *  Created on: Feb 11, 2016
 *      Author: strazce
 */



#define BOOST_TEST_MODULE Vector3D_performance
#include <boost/test/unit_test.hpp>

#include <boost/mpl/back_inserter.hpp>
#include <boost/mpl/insert_range.hpp>
#include <boost/mpl/list.hpp>

#include <chrono>

#include "../../src/include/data/vector.h"

namespace mpl = boost::mpl;

// shorthands for custom Vector types
constexpr static int DIM = 3;
template<typename T> using V3 = opice::data::vector::Vector<T,DIM,true>;

using V3f = V3<float>;

// Types to be used in templated test cases
using float_types = mpl::list<float, double, long double>;
using int_types = mpl::list<short, int, long>;

using arithmetic_types = mpl::insert_range<
                                float_types,
                                mpl::end<float_types>::type,
                                int_types>::type;


using Clock = std::chrono::system_clock;
using Duration = std::chrono::duration<float>;

constexpr static int NUM = 1000000;
constexpr static int REP = 1000;

//TODO: Variable template here. But will it work on icc?
constexpr static double allowed_err = 0.02;

struct Vectors {
    V3f a, b;
    Vectors() : a(1,2,3), b(4.5,5.5,6.5) {
        BOOST_TEST_MESSAGE("setup vectors");
    };
};

template < typename T, typename F >
void static VEC_EQ(V3<T>& v, F x, F y, F z){
    BOOST_CHECK(v.x == x);
    BOOST_CHECK(v.y == y);
    BOOST_CHECK(v.z == z);
}

template <typename T>
void static NOT_SLOWER(T a, T b){
    double perc = static_cast<double>(a)/static_cast<double>(b) - 1;
    BOOST_TEST_MESSAGE("a = " << a << "  slower than   " << " = " << b 
                       << " ??? (tolerance " << allowed_err << ")" );
    BOOST_CHECK(perc < allowed_err);
}

BOOST_FIXTURE_TEST_SUITE(BasicOperations, Vectors)

BOOST_AUTO_TEST_CASE_TEMPLATE(CreateAndAdd, T, float_types){
    auto start = Clock::now();
    V3<T> result = {0,0,0};
    for(int rep=0; rep<REP; ++rep){
        for(int n=0; n<NUM; ++n){
            T i = static_cast<T>(n);
            V3<T> adder(i,i,i);
            result = result + adder;
        }
    }
    Duration vec_time = Clock::now() - start;

    start = Clock::now();
    T aresult[3] = {0,0,0};
    for(int rep=0; rep<REP; ++rep){
        for(int n=0; n<NUM; ++n){
            T i = static_cast<T>(n);
            T df[3] = {i, i, i}; 

            aresult[0] += df[0];
            aresult[1] += df[1];
            aresult[2] += df[2];
        }
    }
    Duration arr_time = Clock::now() - start;

    //TODO: Loop by operator[]
    BOOST_CHECK(result.x == aresult[0]);
    BOOST_CHECK(result.y == aresult[1]);
    BOOST_CHECK(result.z == aresult[2]);

    NOT_SLOWER(vec_time.count(), arr_time.count());
    
}

BOOST_AUTO_TEST_SUITE_END()

