/*
 * vector3d.cpp
 *
 *  Created on: Feb 11, 2016
 *      Author: strazce
 */



#define BOOST_TEST_MODULE Vector2D
#include <boost/test/unit_test.hpp>

#include <boost/mpl/back_inserter.hpp>
#include <boost/mpl/insert_range.hpp>
#include <boost/mpl/list.hpp>

#include "../../src/include/data/vector.h"

namespace mpl = boost::mpl;

// Testing a 2D vector without the SoA container
template<typename T> using V2 = opice::data::vector::Vector<T,2,false>;

using V2f = V2<float>;

using float_types = mpl::list<float, double, long double>;
using int_types = mpl::list<short, int, long>;

using arithmetic_types = mpl::insert_range<
                                float_types,
                                mpl::end<float_types>::type,
                                int_types>::type;

struct Vectors {
    V2f a, b;
    Vectors() :
        a(1,2),
        b(4.5,5.5)
    {
        BOOST_TEST_MESSAGE("setup vectors");
    };
};

template < typename T, typename F >
void static VEC_EQ(V2<T>& v, F x, F y){
    BOOST_CHECK(v.x == x);
    BOOST_CHECK(v.y == y);
}

template<typename T, typename F>
void static VEC_EQ_ALL(V2<T>& v, F x){
    VEC_EQ(v, x, x);
}

BOOST_AUTO_TEST_SUITE(Construction)

BOOST_AUTO_TEST_CASE_TEMPLATE(Construct, T, arithmetic_types){
    V2<T> v(1,2);
    VEC_EQ(v, 1, 2);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(ElementAccess, T, arithmetic_types){
    V2<T> v(0,0);
    v.x = 10; v.y = 20;
    BOOST_CHECK(v[0] == 10);
    BOOST_CHECK(v[1] == 20);
}

BOOST_AUTO_TEST_SUITE_END()



BOOST_FIXTURE_TEST_SUITE(Statics, Vectors)

BOOST_AUTO_TEST_CASE_TEMPLATE(Zeros, T, arithmetic_types){
    V2<T> c = a + V2<T>::Zeros();
    BOOST_CHECK(c == a);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(Ones, T, arithmetic_types){
    V2<T> c = a + V2<T>::Ones();
    VEC_EQ(c, a[0]+1, a[1]+1);
}

BOOST_AUTO_TEST_SUITE_END()



BOOST_FIXTURE_TEST_SUITE(BasicOperations, Vectors)

BOOST_AUTO_TEST_CASE_TEMPLATE(BinaryPlus, T, float_types){
    V2<T> c = a + b;
    VEC_EQ(c, 5.5, 7.5);
}

BOOST_AUTO_TEST_CASE(PlusEquals){
    b += a;
    VEC_EQ(b, 5.5, 7.5);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(BinaryMinus, T, float_types){
    V2<T> c = a - b;
    VEC_EQ_ALL(c, -3.5);
}

BOOST_AUTO_TEST_CASE(MinusEquals){
    b -= a;
    VEC_EQ_ALL(b, 3.5);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(BinaryMultiply, T, float_types){
    V2<T> c = 2*a;
    VEC_EQ(c, 2, 4);
    V2<T> d = a*2;
    VEC_EQ(d, 2, 4);
}

BOOST_AUTO_TEST_CASE(MultiplyEquals){
    a *= 2;
    VEC_EQ(a, 2, 4);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(BinaryDivide, T, float_types){
    V2<T> c = 2*a;
    V2<T> d = c/2;
    VEC_EQ(d, 1, 2);
}

BOOST_AUTO_TEST_CASE(DivideEquals){
    V2f c = 2*a;
    c /= 2;
    VEC_EQ(c, 1, 2);
}

BOOST_AUTO_TEST_SUITE_END()

